﻿using System;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Runtime.CompilerServices;
using WpfModel.Validation;
using ValidationResult = System.Windows.Controls.ValidationResult;

namespace WpfModel.Repository
{
  public abstract class Item : INotifyPropertyChanged
  {
    private Guid _id;
    public Guid Id
    {
      get { return _id; }
      set
      {
        if (ReadOnly)
        {
          return;
        }

        _id = value;
        OnPropertyChanged();
      }
    }

    private int _number;
    public Int32 Number
    {
      get { return _number; }
      set
      {
        if (ReadOnly)
        {
          return;
        }

        // equal value
        if (_number == value)
        {
          return;
        }

        // validation
        var result = ValidateProperty(value);
        CheckValidationResult(result);

        // changing the current value
        _number = value;
        OnPropertyChanged();
      }
    }

    private string _name;
    public String Name
    {
      get { return _name; }
      set
      {
        if (ReadOnly)
        {
          return;
        }

        // equal value
        if (_name == value)
        {
          return;
        }

        // validation
        var result = ValidateProperty(value);
        CheckValidationResult(result);

        // changing the current value
        _name = value;
        OnPropertyChanged();
      }
    }

    public Boolean ReadOnly { get; set; }

    public ValidationResultCollection ValidationResults { get; private set; }

    public event PropertyChangedEventHandler PropertyChanged;

    protected virtual void OnPropertyChanged([CallerMemberName] string propertyName = null)
    {
      PropertyChangedEventHandler handler = PropertyChanged;
      if (handler != null)
      {
        handler(this, new PropertyChangedEventArgs(propertyName));
      }
    }

    protected Item()
    {
      ValidationResults = new ValidationResultCollection();
    }

    protected ValidationResult ValidateProperty(Object value, [CallerMemberName] string propertyName = null)
    {
      switch (propertyName)
      {
        case "Name":
          return ValidateName((String)value);

        case "Number":
          return ValidateNumber((Int32)value);

        default:
          return ValidationResult.ValidResult;
      }
    }

    private ValidationResult ValidateName(String value)
    {
      if (String.IsNullOrEmpty(value))
      {
        var result = new ValidationResultBase(false, "Name cannot be empty!", this);
        ValidationResults.Add("Name", result);
        return result;
      }

      ValidationResults.Remove("Name");
      return ValidationResult.ValidResult;
    }

    private ValidationResult ValidateNumber(Int32 value)
    {
      if (value == 0)
      {
        var result = new ValidationResultBase(false, "Number has to be larger than 0!", this);
        ValidationResults.Add("Name", result);
        return result;
      }

      ValidationResults.Remove("Number");
      return ValidationResult.ValidResult;
    }

    protected void CheckValidationResult(ValidationResult result)
    {
      if (result.IsValid)
      {
        return;
      }

      throw new ValidationException(result.ErrorContent.ToString());
    }

    public TItem CreateCopy<TItem>()
      where TItem : Item, new()
    {
      Item item = new TItem();
      CreateCopyOverride(ref item);
      return (TItem)item;
    }

    protected virtual void CreateCopyOverride(ref Item item)
    {
      item.Id = Id;
      item.Name = Name;
      item.Number = Number;
    }

    public void Merge<TItem>(Item copy)
      where TItem : Item
    {
      MergeOverride(copy);
    }

    protected virtual void MergeOverride(Item copy)
    {
      Id = copy.Id;
      Name = copy.Name;
      Number = copy.Number;
    }
  }
}
