﻿using System;
using System.ComponentModel;
using System.Linq;
using Microsoft.Practices.Prism.Commands;
using Microsoft.Practices.Prism.Mvvm;
using WpfModel;
using WpfModel.Repository;

namespace WpfViewModel.Base
{
  public sealed class MainViewModel : BindableBase
  {
    private readonly GenericRepository<Person> _persons = new GenericRepository<Person>();
    public GenericRepository<Person> Persons
    {
      get { return _persons; }
    }

    private Person _selectedItem;
    public Person SelectedItem
    {
      get { return _selectedItem; }
      set
      {
        if (_selectedItem != null)
        {
          _selectedItem.PropertyChanged -= SelectedItemPropertyChanged;
        }

        SetProperty(ref _selectedItem, value);

        if (_selectedItem != null)
        {
          _selectedItem.PropertyChanged += SelectedItemPropertyChanged;
        }

        InvalidateCommands();
      }
    }

    private Boolean _isEditing;
    public Boolean IsEditing
    {
      get { return _isEditing; }
      set
      {
        if (SetProperty(ref _isEditing, value))
        {
          EditCommand.RaiseCanExecuteChanged();
          SaveCommand.RaiseCanExecuteChanged();
          CancelCommand.RaiseCanExecuteChanged();
          RemoveCommand.RaiseCanExecuteChanged();
        }
      }
    }

    private void InvalidateCommands()
    {
      ClearCommand.RaiseCanExecuteChanged();
      AddCommand.RaiseCanExecuteChanged();
      RemoveCommand.RaiseCanExecuteChanged();
      EditCommand.RaiseCanExecuteChanged();
      SaveCommand.RaiseCanExecuteChanged();
      CancelCommand.RaiseCanExecuteChanged();
    }

    private void SelectedItemPropertyChanged(object sender, PropertyChangedEventArgs e)
    {
      InvalidateCommands();
    }

    public DelegateCommand AddCommand { get; private set; }
    public DelegateCommand RemoveCommand { get; private set; }
    public DelegateCommand ClearCommand { get; private set; }
    public DelegateCommand EditCommand { get; private set; }
    public DelegateCommand SaveCommand { get; private set; }
    public DelegateCommand CancelCommand { get; private set; }

    public MainViewModel()
    {
      Persons.Add(new Person()
      {
        Name = "Mister",
        Surname = "Batman",
        Address = "Gotham City",
        DateOfBirth = new DateTime(2000, 1, 10),
        IsChanged = false
      });
      Persons.Add(new Person()
      {
        Name = "Mister",
        Surname = "Spiderman",
        Address = "New York",
        DateOfBirth = new DateTime(1000, 1, 10),
        IsChanged = false
      });

      ClearCommand = new DelegateCommand(Clear);
      AddCommand = new DelegateCommand(AddPerson, CanAddPerson);
      RemoveCommand = new DelegateCommand(RemovePerson, CanRemovePerson);
      EditCommand = new DelegateCommand(EditPerson, CanEditPerson);
      SaveCommand = new DelegateCommand(SavePerson, CanSavePerson);
      CancelCommand = new DelegateCommand(CancelEditation, CanCancelEditation);

      IsEditing = false;
      Clear();
    }

    private bool CanCancelEditation()
    {
      return IsEditing;
    }

    private void CancelEditation()
    {
      IsEditing = false;
      SelectedItem.IsChanged = false;
    }

    private bool CanSavePerson()
    {
      return IsEditing && SelectedItem != null && SelectedItem.IsChanged && IsPersonValid();
    }

    private void SavePerson()
    {
      IsEditing = false;
      InvalidateCommands();
    }

    private bool CanEditPerson()
    {
      return SelectedItem != null && !IsEditing;
    }

    private void EditPerson()
    {
      IsEditing = true;
      InvalidateCommands();
    }

    private void Clear()
    {
      SelectedItem = new Person();
    }

    private bool IsPersonValid()
    {
      return SelectedItem != null && SelectedItem.IsValid();
    }

    private bool ContainsItem()
    {
      return _persons.Items.Contains(SelectedItem);
    }

    private bool CanAddPerson()
    {
      return !IsEditing;
    }

    private void AddPerson()
    {
      var person = new Person();
      Persons.Add(person);
      SelectedItem = person;

      EditPerson();
      InvalidateCommands();
    }

    private bool CanRemovePerson()
    {
      return IsEditing && SelectedItem != null;
    }

    private void RemovePerson()
    {
      if (ContainsItem())
      {
        Persons.Delete(SelectedItem);
        IsEditing = false;

        SelectedItem = Persons.Items.FirstOrDefault();
      }

      InvalidateCommands();
    }
  }
}
